{% extends "base.html" %}

{% block title %}
    Errore
{% endblock %}

{% block content %}
    <div class="error">
        <div class="dbox error-dbox">
            <div class="dbox-top">
                <span class="left">Errore!</span>
            </div>
            <div class="dbox-bot">
                <h1>
                    {{ title }}
                </h1>
                <p>
                    {{ reason }}
                </p>
            </div>
        </div>
    </div>
{% endblock %}