{% extends "base.html" %}

{% block title %}
    Home
{% endblock %}

{% block content %}
    <div class="dbox">
        <div class="dbox-top">
            <span class="left">Benvenuto!</span>
        </div>
        <div class="dbox-bot">
            <p>
                Benvenuto al sito web della <b>Royal Games</b>!
            </p>
            <p>
                Siamo una piccola community amichevole principalmente dedicata al PC gaming.
            </p>
            <p>
                Vienici a trovare su <a href="https://discord.gg/UpuU9Y4">Discord</a> o <a href="https://steamcommunity.com/groups/royalgamescastle">Steam</a>!
            </p>
        </div>
    </div>
{% endblock %}